import { ModuleWithProviders, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

import { DEFAULT_THEME } from './styles/theme.default';
import { COSMIC_THEME } from './styles/theme.cosmic';
import { CORPORATE_THEME } from './styles/theme.corporate';
import { DARK_THEME } from './styles/theme.dark';
import { FormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import { SearchModule } from 'app/features/search/search.module';
import { FooterComponent } from './components/footer/footer.component';
import { HeaderComponent } from './components/header/header.component';
import { SharedModule } from 'app/shared/shared.module';
import { NbJSThemeOptions, NbThemeModule } from '@nebular/theme';
import { OneColumnLayoutComponent } from './layouts/one-column/one-column.layout';
import { ThreeColumnsLayoutComponent } from './layouts/three-columns/three-columns.layout';
import { TwoColumnsLayoutComponent } from './layouts/two-columns/two-columns.layout';
import { CoursesModule } from 'app/features/courses/courses.module';
import { TermSelectionComponent } from './components/term-selection/term-selection.component';
import { SideNavComponent } from './components/side-nav/side-nav.component';

export const SUBATO_THEME = {
  name: 'subato',
  base: 'subato',
  variables: {
    
  },
} as NbJSThemeOptions;

@NgModule({
  declarations: [
    HeaderComponent,
    FooterComponent,
    OneColumnLayoutComponent,
    ThreeColumnsLayoutComponent,
    TwoColumnsLayoutComponent,
    TermSelectionComponent,
    SideNavComponent
  ],
  imports: [
    CommonModule, 
    FormsModule, 
    RouterModule, 
    SharedModule,
    SearchModule,
    CoursesModule
  ],
  exports: [
    CommonModule,
    OneColumnLayoutComponent,
    ThreeColumnsLayoutComponent,
    TwoColumnsLayoutComponent
  ],
})
export class CoreModule {
  static forRoot(): ModuleWithProviders<CoreModule> {
    return {
      ngModule: CoreModule,
      providers: [
        ...NbThemeModule.forRoot(
          {
            name: 'subato',
          },
          [ DEFAULT_THEME, COSMIC_THEME, CORPORATE_THEME, DARK_THEME, SUBATO_THEME ],
        ).providers,
      ],
    };
  }
}
