import { Injectable } from '@angular/core';
import { CanActivateChild, ActivatedRouteSnapshot, RouterStateSnapshot, UrlTree, Router } from '@angular/router';
import { AppStateService } from 'app/core/services/app.state.service';
import { AuthService } from 'app/core/services/auth.service';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest, from, Observable, of } from 'rxjs';
import { catchError, filter, flatMap, map } from 'rxjs/operators';
import { KeycloakService } from 'keycloak-angular';
import { CourseService } from 'app/features/courses/services/course.service';
import { Term } from 'app/features/modules/models/term';

@Injectable({
  providedIn: 'root'
})
export class AuthGuard implements CanActivateChild {
  constructor(
    private authService: AuthService,
    private router: Router,
    private notificationService: NotificationService,
    private appStateService: AppStateService,
    private courseService: CourseService,
    private keycloakService: KeycloakService) { }

  canActivateChild(
    next: ActivatedRouteSnapshot,
    state: RouterStateSnapshot): Observable<boolean | UrlTree> | Promise<boolean | UrlTree> | boolean | UrlTree {
    return this.appStateService.init$.pipe(
      filter(init => init),
      flatMap(init => combineLatest(this.authService.principal$, this.appStateService.prevUrl$)),
      flatMap(async ([principal, prevUrl]) => {
        if (!next.data.courseMenuEnabled) {
          this.appStateService.setCourse(null);
        }

        if (next.data.authRequired && !principal) {
          await this.keycloakService.login({
            redirectUri: window.location.origin + state.url
          });
          return;
        }

        return true;
      })
    );
  }
}
