import { Injectable } from '@angular/core';
import { NavigationEnd, Router } from '@angular/router';
import { NbMediaBreakpointsService, NbMenuItem, NbThemeService } from '@nebular/theme';
import { combineLatest, ReplaySubject } from 'rxjs';
import { Observable } from 'rxjs/Observable';
import { takeUntil } from 'rxjs/operators';
import { map } from 'rxjs/operators';
import { AuthService } from './auth.service';
import { BaseComponent } from '../../shared/components/base/base.component';
import { AppStateService } from './app.state.service';
import { Course } from 'app/features/courses/models/course';
import { Term } from 'app/features/modules/models/term';
import { SubatoPrincipal } from '../models/principal/subato-principal';
import { MainMenu } from './main-menu';
import { CourseMenu } from './course-menu';

export class SidebarNav {
  mainMenu: NbMenuItem[] = [];
  courseMenu: NbMenuItem[] = [];

  items: NbMenuItem[] = [];

  constructor(mainMenu: NbMenuItem[], courseMenu: NbMenuItem[]) {
    this.mainMenu = mainMenu;
    this.courseMenu = courseMenu;
    this.merge();
  }

  private merge() {
    this.items = this.mainMenu.concat(this.courseMenu);
  }
}

@Injectable({
  providedIn: 'root'
})
export class SidebarNavService extends BaseComponent {
  private _nav: ReplaySubject<SidebarNav>
  nav$: Observable<SidebarNav>

  constructor(
    private authService: AuthService,
    private appStateService: AppStateService,
    private breakpointService: NbMediaBreakpointsService,
    private themeService: NbThemeService,
    router: Router) {
    super();

    this._nav = new ReplaySubject<SidebarNav>();
    this.nav$ = this._nav.asObservable();

    combineLatest(router.events, this.authService.principal$, this.appStateService.term$, this.appStateService.courseId$, this.compactMenu())
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(([val, principal, term, courseId, menuIsCompact]) => {
        if ((<any>val).routerEvent instanceof NavigationEnd) {
          var nav = this.getMenu(principal, term, courseId, (<any>val).routerEvent.url, menuIsCompact);
          this._nav.next(nav);
        }
      });
  }

  public getMenu(principal: SubatoPrincipal | null, term: Term | null, course: Course | null, url: string, menuIsCompact: boolean): SidebarNav {
    let mainMenu: NbMenuItem[] = menuIsCompact 
      ? MainMenu.getMenu(this.appStateService.currentUrl, course, principal) 
      : [];
    let courseMenu = course != null 
      ? CourseMenu.getMenu(this.appStateService.currentUrl, course, principal) 
      : [];
    return new SidebarNav(mainMenu, courseMenu);
  }

  compactMenu(): Observable<boolean> {
    const { lg } = this.breakpointService.getBreakpointsMap();
    return this.themeService.onMediaQueryChange()
      .pipe(
        map(([, currentBreakpoint]) => currentBreakpoint.width < lg),
        takeUntil(this.unsubscribe)
      );
  }
}