import { HttpClient, HttpParams } from "@angular/common/http";
import { Injectable } from "@angular/core";
import { BackendError } from "app/shared/models/backend-error";
import { DataQueryParams } from "app/shared/models/data-query-params";
import { Page } from "app/shared/models/page";
import { BaseService } from "app/shared/services/base.service";
import { environment } from "environments/environment";
import { Observable, of } from "rxjs";
import { catchError, map, flatMap } from "rxjs/operators";
import { User } from "../models/user";

export interface UserParams {
    q?: string;
    projection?: DataQueryParams;
}

@Injectable({
    providedIn: 'root'
})
export class UserService extends BaseService {
    constructor(private http: HttpClient) {
        super();
    }

    getById(id: number): Observable<User> {
        return this.http.get<User>(`${environment.backendUrl}/user/${id}`).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(user => new User(user))
        );
    }

    findAll(params?: UserParams): Observable<Page<User>> {
        let httpParams = DataQueryParams.toHttpParams(params?.projection)

        if(params?.q != null) {
            httpParams = httpParams.set('q', params.q);
        }

        return this.http.get<Page<User>>(`${environment.backendUrl}/user`, {
            params: httpParams,
        }).pipe(
            catchError(e => {
                throw BackendError.parse(e);
            }),
            map(page => new Page<User>(page, item => new User(item)))
        )
    }
}