import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { NotificationService } from 'app/shared/services/notification.service';
import { Subject } from 'rxjs';
import { debounceTime, takeUntil } from 'rxjs/operators';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { UntypedFormBuilder } from '@angular/forms';
import { User } from 'app/core/models/user';
import { BackendError } from 'app/shared/models/backend-error';
import { Course } from '../../models/course';
import { CourseService } from '../../services/course.service';


@Component({
  selector: 'ngx-course-participants-list',
  templateUrl: './course-participants-list.component.html',
  styleUrls: ['./course-participants-list.component.scss']
})
export class CourseParticipantsListComponent extends BaseComponent implements OnInit {
  @Input() course: Course;
  loading: boolean = true;
  participants: User[] = [];
  allParticipants: User[] = [];

  error: BackendError;
  query: string;
  private queryChanged: Subject<string> = new Subject<string>();

  constructor(
    private courseService: CourseService,
    private notificationService: NotificationService,
    private route: ActivatedRoute,
    private authService: AuthService,
    private router: Router,
    private fb: UntypedFormBuilder
  ) {
    super();
  }

  ngOnInit(): void {
    this.courseService.getParticipants(this.course.id)
      .pipe(takeUntil(this.unsubscribe))
      .subscribe(participants => {
        this.allParticipants = participants;
        this.participants = this.allParticipants;
        this.loading = false;
      }, error => {
        if (BackendError.isBackendError(error)) {
          this.error = error;
          this.loading = false;
          this.notificationService.showError(
            error.format(
              "Fehler beim Laden der Teilnehmer",
            )
          );
        } else {
          throw error;
        }
      });

    this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(500),
      )
      .subscribe(query => {
        this.participants = this.allParticipants.filter(
          p => p.name.toLowerCase().indexOf(query.toLowerCase()) != -1 ||
               (p.loginName != null && p.loginName.toLowerCase().indexOf(query.toLowerCase()) != -1)
        );
      });
  }


  onChange(query) {
    this.queryChanged.next(query);
  }
}
