import { ApplicationRef, ChangeDetectorRef, Component, OnInit, ViewChild } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { NbDialogService, NbStepChangeEvent, NbStepperComponent } from '@nebular/theme';
import { AuthService } from 'app/core/services/auth.service';
import { TaskService } from 'app/features/tasks/services/task.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { Breadcrumb } from 'app/shared/components/breadcrumbs/breadcrumbs.component';
import { ErrorContainerConfig } from 'app/shared/components/error-container/error-container.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { takeUntil, flatMap } from 'rxjs/operators';
import { Misconception } from '../../models/misconception';
import { MisconceptionService } from '../../services/misconception.service';

@Component({
  selector: 'ngx-misconception-detail',
  templateUrl: './misconception-detail.component.html',
  styleUrls: ['./misconception-detail.component.scss']
})
export class MisconceptionDetailComponent extends BaseComponent implements OnInit {
  loading: boolean;
  misconception: Misconception;
  liked: boolean | null = null;
  selectedIndex: number = 0;

  error: BackendError;
  errorConfig: ErrorContainerConfig = {
    notFound: {
      message: 'Diese Fehlvorstellung konnten wir leider nicht finden.'
    }
  };

  breadcrumbs: Breadcrumb[] = [];
  showContext = false;
  
  constructor(
    private authService: AuthService,
    private misconceptionService: MisconceptionService,
    private dialogService: NbDialogService,
    private router: Router,
    private route: ActivatedRoute,
    private notificationService: NotificationService,
  ) {
    super();
  }

  rate(val: boolean) {
    this.liked = val;
  }
  
  ngOnInit(): void {
    this.loading = true;
    this.route.paramMap
      .pipe(
        takeUntil(this.unsubscribe),
        flatMap((params) => {
          let misconceptionId = params.get("misconception_id");
          return this.misconceptionService.getById(misconceptionId); 
        }),
        ).subscribe(misconception => {
          this.misconception = misconception;
          this.loading = false;
        }, error => {
          if (BackendError.isBackendError(error)) {
            this.error = error;
            this.loading = false;
          } else {
            throw error;
          }
      });
  }

  stepChanged(event: NbStepChangeEvent) {
    this.selectedIndex = event.index;
  }
}
