import { Component, OnInit } from '@angular/core';
import { Term } from '../../models/term';
import { TermService } from '../../services/term.service';
import { Router, ActivatedRoute } from '@angular/router';
import { AuthService } from 'app/core/services/auth.service';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { BackendError } from 'app/shared/models/backend-error';
import { NotificationService } from 'app/shared/services/notification.service';
import { combineLatest } from 'rxjs';
import { takeUntil, flatMap, filter } from 'rxjs/operators';
import { User } from 'app/core/models/user';
import { SubatoPrincipal } from 'app/core/models/principal/subato-principal';

@Component({
  selector: 'ngx-terms',
  templateUrl: './terms.component.html',
  styleUrls: ['./terms.component.scss']
})
export class TermsComponent extends BaseComponent implements OnInit {
  loading: boolean = true;
  error: BackendError;
  principal: SubatoPrincipal;

  constructor(
    private router: Router,
    private authService: AuthService,
    private notificationService: NotificationService,
    private route: ActivatedRoute) {
    super();
  }

  ngOnInit(): void {
    this.loading = true;
    combineLatest(this.route.paramMap, this.authService.principal$)
      .pipe(
        takeUntil(this.unsubscribe),
      )
      .subscribe(([params, principal]) => {
        this.principal = principal;
        this.loading = false;
      });
  }

}
