import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Router } from '@angular/router';
import { Subject } from 'rxjs';
import { BaseComponent } from 'app/shared/components/base/base.component';
import { takeUntil, debounceTime } from 'rxjs/operators';
import { CourseResult } from 'app/features/courses/models/course-result';
import { ExerciseResult } from 'app/features/courses/models/exercise-result';

@Component({
  selector: 'ngx-course-result',
  templateUrl: './course-result.component.html',
  styleUrls: ['./course-result.component.scss']
})
export class CourseResultComponent extends BaseComponent implements OnInit {
  @Input() courseResult: CourseResult;
  @Input() showFilter: boolean;
  @Input() hideInvisible: boolean;

  @Input() set exerciseResults(val: ExerciseResult[]) {
    this._exerciseResults = val;
    this.applyFilter();
  }

  get exerciseResults(): ExerciseResult[] {
    return this._exerciseResults;
  }

  @Output() hideInvisibleChanged: EventEmitter<boolean> = new EventEmitter<boolean>();

  _exerciseResults: ExerciseResult[] = [];
  filteredExerciseResults: ExerciseResult[] = [];
  query: string;
  private queryChanged: Subject<string> = new Subject<string>();

  constructor(private router: Router) {
    super();
  }

  ngOnInit(): void {
    this.queryChanged
      .pipe(
        takeUntil(this.unsubscribe),
        debounceTime(500),
      )
      .subscribe(query => {
        this.query = query;
        this.applyFilter();
      });
  }

  onHideInvisibleChanged(val: boolean) {
    this.hideInvisible = val;
    this.hideInvisibleChanged.emit(val);
  }

  onQueryChange(query) {
    this.queryChanged.next(query);
  }

  applyFilter() {
    this.filteredExerciseResults = this.query 
      ? this.exerciseResults.filter(
          r => r.exercise.name.toLowerCase().indexOf(this.query.toLowerCase()) !== -1 ||
          r.taskInstances.some(ti => ti.task.name.toLocaleLowerCase().indexOf(this.query.toLowerCase()) !== -1)
      )
      : this.exerciseResults;
  }
}
