import { NgModule } from '@angular/core';
import { Routes, RouterModule, ActivatedRouteSnapshot } from '@angular/router';
import { User } from '../../core/models/user';
import { TaskPoolsComponent } from './pages/task-pools/task-pools.component';
import { TaskComponent } from './pages/task/task.component';
import { TaskPoolComponent } from './pages/task-pool/task-pool.component';
import { LanguagesComponent } from './pages/languages/languages.component';
import { NewLanguageComponent } from './pages/new-language/new-language.component';
import { EditLanguageComponent } from './pages/edit-language/edit-language.component';
import { EditTaskPoolComponent } from './pages/edit-task-pool/edit-task-pool.component';
import { NewTaskPoolComponent } from './pages/new-task-pool/new-task-pool.component';
import { TaskPool } from './models/task-pool';
import { Task } from './models/task';

export const routes: Routes = [
  {
    path: 'task_pools',
    component: TaskPoolsComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'pools',
        label: 'Aufgaben-Pools' 
      }
    }
  },
  {
    path: 'task_pools/new',
    component: NewTaskPoolComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'newPool',
        label: 'Erstellen' 
      }
    }
  },
  {
    path: 'task_pools/:pool_id/edit',
    component: EditTaskPoolComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'editPool',
        label: 'Bearbeiten' 
      }
    }
  },
  {
    path: 'task_pools/:pool_id',
    component: TaskPoolComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'pool',
        label: (pool: TaskPool) => `${pool.name}` 
      }
    }
  },
  {
    path: 'tasks/:task_id',
    component: TaskComponent,
    data: {
      authRequired: true,
      breadcrumb: {
        alias: 'task',
        label: (task: Task) => `${task.name}` 
      }
    }
  },
  {
    path: 'language',
    component: LanguagesComponent,
    data: {
      authRequired: true
    }
  },
  {
    path: 'language/new',
    component: NewLanguageComponent,
    data: {
      authRequired: true
    }
  },
  {
    path: 'language/:language_id/edit',
    component: EditLanguageComponent,
    data: {
      authRequired: true
    }
  },
];

@NgModule({
  exports: [RouterModule]
})
export class TasksRoutingModule { }
