package de.hsrm.sls.subato.aor.core;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;

// gradle bootRun --args='import:core /home/fischer/Desktop/stammdaten_v3.json'
@Component
public class CoreDataImportRunner implements CommandLineRunner {
    @Autowired
    CoreDataImportManager coreDataImportManager;

    Logger logger = LoggerFactory.getLogger(CoreDataImportRunner.class);

    @Override
    public void run(String... args) {
        if (args.length < 1 || !args[0].equals("import:core")) return;

        var filePath = args[1];
        var f = new File(filePath);
        logger.info("Importing core data: %s".formatted(filePath));

        try {
            var content = Files.readString(f.toPath(), StandardCharsets.UTF_8);
            coreDataImportManager.importFromJson(content);
        } catch (IOException e) {
            logger.error("File %s not readable".formatted(filePath));
        }
        logger.info("Finished importing core data");

        System.exit(0);
    }
}
