package de.hsrm.sls.subato.auth.policy.courses;

import de.hsrm.sls.subato.auth.Role;
import de.hsrm.sls.subato.auth.policy.CheckPolicyRequest;
import de.hsrm.sls.subato.auth.policy.PolicyRule;
import de.hsrm.sls.subato.auth.policy.SubatoPrincipalSpecifications;
import de.hsrm.sls.subato.auth.principal.SubatoPrincipal;
import de.hsrm.sls.subato.courses.course.Course;
import de.hsrm.sls.subato.courses.course.CourseVisibility;
import de.hsrm.sls.subato.courses.course.Course_;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Component;

/**
 * The current user is allowed to view courses
 */
@Component
public class CourseViewPolicy implements PolicyRule {
    @Override
    public boolean check(CheckPolicyRequest request) {
        if(!request.matches(CoursesAction.COURSE_VIEW)) {
            return false;
        }

        var course = (Course) request.getObj();
        return course.getVisibility() == CourseVisibility.PUBLIC ||
                course.getVisibility() == CourseVisibility.INTERNAL && request.getSubject() != null ||
                course.getVisibility() == CourseVisibility.PARTICIPANTS && request.getSubject() != null && request.getSubject().isParticipant(course.getId()) ||
                (request.getSubject() != null && (request.getSubject().hasRole(Role.Staff) || request.getSubject().hasAnyGroup(course.getId())));
    }


    @Override
    public Specification checkAsSpec(CheckPolicyRequest request) {
        if(!request.matches(CoursesAction.COURSE_VIEW)) {
            return Specification.where(null);
        }

        return isVisible(request.getSubject());
    }

    /**
     * If this logic is changed, it has to be changed in
     * {@link de.hsrm.sls.subato.auth.policy.exercises.ExerciseViewPolicy#isCourseVisible(SubatoPrincipal)} too.
     *
     * sorry.
     */
    public static Specification<Course> isVisible(SubatoPrincipal principal) {
        return (ex, cq, cb) -> {
            var participantInBuilder = SubatoPrincipalSpecifications.participantIn(principal, cb, ex.get(Course_.id));
            var courseStaffInBuilder = SubatoPrincipalSpecifications.courseStaffIn(principal, cb, ex.get(Course_.id));

            return cb.or(
                    cb.equal(ex.get(Course_.visibility), CourseVisibility.PUBLIC),
                    cb.and(
                            cb.equal(ex.get(Course_.visibility), CourseVisibility.INTERNAL),
                            cb.equal(cb.literal(true), principal != null)
                    ),
                    cb.and(
                            cb.equal(ex.get(Course_.visibility), CourseVisibility.PARTICIPANTS),
                            cb.equal(cb.literal(true), principal != null),
                            participantInBuilder
                    ),
                    cb.or(
                            cb.equal(cb.literal(true), principal != null && principal.hasRole(Role.Staff)),
                            courseStaffInBuilder
                    )
                );
        };
    }
}
