package de.hsrm.sls.subato.courses.result;

import de.hsrm.sls.subato.exercises.exercise.AssessmentMode;
import de.hsrm.sls.subato.exercises.assessment.AssessmentProgress;

import java.util.List;

public class CourseResult {
    private List<ExerciseResult> exerciseResults;

    public CourseResult(List<ExerciseResult> exerciseResults) {
        this.exerciseResults = exerciseResults;
    }

    public List<ExerciseResult> getExerciseResults() {
        return exerciseResults;
    }

    public void setExerciseResults(List<ExerciseResult> exerciseResults) {
        this.exerciseResults = exerciseResults;
    }

    public long getExercisesTotal() {
        return exerciseResults.stream().count();
    }

    public long getExercisesBinaryTotal() {
        return exerciseResults.stream().filter(e -> e.getExercise().getAssessmentMode() == AssessmentMode.Binary).count();
    }

    public long getPassedTotal() {
        return exerciseResults.stream()
                .filter(e -> e.getExercise().getAssessmentMode() == AssessmentMode.Binary)
                .filter(e -> e.getSubmission() != null)
                .filter(e -> e.getSubmission().getPassed() != null && e.getSubmission().getPassed())
                .count();
    }

    public long getNotPassedTotal() {
        return exerciseResults.stream()
                .filter(e -> e.getExercise().getAssessmentMode() == AssessmentMode.Binary)
                .filter(e -> e.getSubmission() != null)
                .filter(e -> e.getSubmission().getPassed() != null && !e.getSubmission().getPassed())
                .count();
    }

    public long getUndecidedTotal() {
        return exerciseResults.stream()
                .filter(e -> e.getExercise().getAssessmentMode() == AssessmentMode.Binary)
                .filter(e -> e.getSubmission() != null)
                .filter(e -> e.getSubmission().getPassed() == null)
                .count();
    }

    public long getReachedPointsTotal() {
        return exerciseResults.stream()
                .filter(e -> e.getSubmission() != null)
                .map(e -> e.getSubmission().getTotalPoints())
                .reduce(0, (sum, c) -> sum + c);
    }

    public long getReachablePointsTotal() {
        return exerciseResults.stream()
                .map(e -> e.getExercise().getReachablePoints())
                .reduce(0, (sum, c) -> sum + c);
    }

    public long getTestsPassedTotal() {
        return exerciseResults.stream().map(e -> e.getTestsPassed()).reduce(0, (sum, c) -> sum + c);
    }

    public long getTestsTotal() {
        return exerciseResults.stream().map(e -> e.getTestsTotal()).reduce(0, (sum, c) -> sum + c);
    }

    public long getStyleErrorsTotal() {
        return exerciseResults.stream().map(e -> e.getStyleErrorsTotal()).reduce(0, (sum, c) -> sum + c);
    }

    public long getLeaksTotal() {
        return exerciseResults.stream().map(e -> e.getLeaksTotal()).reduce(0, (sum, c) -> sum + c);
    }
}
