package de.hsrm.sls.subato.dashboard;

import de.hsrm.sls.subato.auth.principal.PrincipalService;
import de.hsrm.sls.subato.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Optional;

@RestController
@Tag(name = "Querschnitt")
public class DashboardController {
    @Autowired
    DashboardStateMapper dashboardStateMapper;

    @Autowired
    DashboardService dashboardService;

    @Autowired
    PrincipalService principalService;

    @Autowired
    UserService userService;

    @GetMapping("/dashboard")
    @PreAuthorize("isAuthenticated()")
    @Operation(summary = "Querschnittliche Aggregation von relevanten Daten für Benutzer, wie z.B. deren Kursen, die Anwesenheit darin, Fortschritt, Bewertungsinformationen etc.")
    DashboardStateDto dashboard(@RequestParam Optional<Long> termId) {
        var principal = principalService.getPrincipal();
        var dashboard = dashboardService.generateDashboard(termId.orElse(null), principal);

        return dashboardStateMapper.map(dashboard);
    }
}
