package de.hsrm.sls.subato.seed.course;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import de.hsrm.sls.subato.courses.course.*;
import de.hsrm.sls.subato.modules.progmodule.ProgramModuleRepository;
import de.hsrm.sls.subato.modules.term.TermRepository;
import de.hsrm.sls.subato.seed.SeedingException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;

@Service
@Transactional
public class CourseSeedService {
    private static Logger logger = LoggerFactory.getLogger(CourseSeedService.class);

    @Autowired
    CourseRepository courseRepository;

    @Autowired
    TermRepository termRepository;

    @Autowired
    ProgramModuleRepository programModuleRepository;

    @Autowired
    CourseService courseService;


    public void seedCourses(File seedDir, ObjectMapper objectMapper) {
        try {
            var coursesPath = seedDir.toPath().resolve("courses.json");
            var json = Files.readString(coursesPath, StandardCharsets.UTF_8);
            var courseSeeds = objectMapper.readValue(json, new TypeReference<List<CourseSeed>>() {});

            logger.debug("Seeding %s courses...".formatted(courseSeeds.size()));

            for (var courseSeed : courseSeeds) {
                if (courseSeed.getId() != null && courseRepository.existsBySourceId(courseSeed.getId())) {
                    continue;
                }

                var request = new CreateCourseRequestBuilder()
                        .setSourceId(courseSeed.getId())
                        .setName(courseSeed.getName())
                        .setTermId(courseSeed.getTermId())
                        .setShortName(courseSeed.getShortName())
                        .setShowEmails(true)
                        .setVisibility(courseSeed.getVisibility())
                        .setSubmitPolicy(courseSeed.getSubmitPolicy());

                var modules = programModuleRepository.findByAorIdIn(courseSeed.getModuleIds());
                request = request.setModuleIds(modules.stream().map(m -> m.getId()).toList());

                if(courseSeed.getTermId() != null) {
                    var term = termRepository.findBySourceId(courseSeed.getTermId()).orElse(null);
                    request = request.setTermId(term.getId());
                }

                courseService.create(request.build(), null);
            }
        } catch (IOException e) {
            throw new SeedingException(e);
        }
    }
}
