package de.hsrm.sls.subato.solutions.solution;

import de.hsrm.sls.subato.Tags;
import de.hsrm.sls.subato.utils.PrepareSubatoDataDir;
import de.hsrm.sls.subato.utils.SubatoTmpDirCleanup;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;

import java.util.Optional;
import java.util.UUID;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * Integration Test für {@link SolutionController#getFile(Long, UUID, Optional)}
 */
@SpringBootTest
@AutoConfigureMockMvc
@DirtiesContext(classMode = DirtiesContext.ClassMode.BEFORE_EACH_TEST_METHOD)
@AutoConfigureTestDatabase(replace = AutoConfigureTestDatabase.Replace.ANY)
@Tag(Tags.INTEGRATION)
public class GetFileTest {
    @Autowired
    private MockMvc mockMvc;

    @BeforeAll
    public static void beforeAll(@Autowired PrepareSubatoDataDir dataDirCleanup, @Autowired SubatoTmpDirCleanup tmpDirCleanup) {
        dataDirCleanup.exec();
        tmpDirCleanup.exec();
    }

    @Test
    public void shouldDenyGuests() throws Exception {
        this.mockMvc.perform(get("/solution/1/file/b3e5a0ff-94e9-40e9-9633-8fd966ff7010")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isUnauthorized());
    }

    @Test
    @WithUserDetails("US_C1A")
    public void shouldAllowCourseAdmins() throws Exception {
        this.mockMvc.perform(get("/solution/1/file/b3e5a0ff-94e9-40e9-9633-8fd966ff7010")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk());
    }
}
